#cython: language_level=3
 
from .capi.obiview      cimport obi_column_get_obiseq_with_elt_name_in_view, \
                                obi_column_get_obiseq_with_elt_idx_in_view, \
                                obi_column_set_obiseq_with_elt_name_in_view, \
                                obi_column_set_obiseq_with_elt_idx_in_view
from .capi.obialign     cimport obi_align_one_column
from .capi.obierrno     cimport obi_errno
from .capi.obitypes     cimport OBISeq_NA, const_char_p

from ._obidms           cimport OBIView

from obitools3.utils cimport str2bytes, bytes2str

from libc.stdlib cimport free


cdef class OBIDMS_column_seq(OBIDMS_column):

    cpdef object get_line(self, index_t line_nb):
        cdef char*  value
        cdef object result
        value = obi_column_get_obiseq_with_elt_idx_in_view(self.view.pointer, (self.pointer)[0], line_nb, 0)
        if obi_errno > 0 :
            raise IndexError(line_nb)
        if value == OBISeq_NA :
            result = None
        else :
            try:
                result = <bytes> value
            finally:
                free(value)
        return result

    cpdef set_line(self, index_t line_nb, object value):
        cdef bytes value_b

        if value is None :
            value_b = OBISeq_NA
        elif isinstance(value, bytes) :
            value_b = value
        elif isinstance(value, str) :
            value_b = str2bytes(value)
        else:
            raise TypeError('Sequence value must be of type Bytes, Str or None')

        if obi_column_set_obiseq_with_elt_idx_in_view(self.view.pointer, (self.pointer)[0], line_nb, 0, value_b) < 0:
            raise Exception("Problem setting a value in a column")
        else :
            if obi_column_set_obiseq_with_elt_idx_in_view(self.view.pointer, (self.pointer)[0], line_nb, 0, str2bytes(value)) < 0:
                raise Exception("Problem setting a value in a column")
    
    # TODO choose alignment type (lcs or other) with supplementary argument
    cpdef align(self, 
                OBIView score_view,
                OBIDMS_column score_column, 
                double threshold = 0.0, 
                bint normalize = True, 
                int reference = 0,      # TODO
                bint similarity_mode = True):
        if (obi_align_one_column(self.view.pointer, (self.pointer)[0], score_view.pointer, (score_column.pointer)[0], threshold, normalize, reference, similarity_mode) < 0) :
            raise Exception("An error occurred while aligning sequences")

        

cdef class OBIDMS_column_multi_elts_seq(OBIDMS_column_multi_elts):

    cpdef object get_item(self, index_t line_nb, str element_name):
        cdef char*  value
        cdef object result
        value = obi_column_get_obiseq_with_elt_name_in_view(self.view.pointer, (self.pointer)[0], line_nb, str2bytes(element_name))
        if obi_errno > 0 :
            raise IndexError(line_nb, element_name)
        if value == OBISeq_NA :
            result = None
        else :
            try:
                result = <bytes> value
            finally:
                free(value)
        return result

    
    cpdef object get_line(self, index_t line_nb) :
        cdef char*   value
        cdef object  value_in_result
        cdef dict    result
        cdef index_t i
        cdef bint    all_NA
        result = {}
        all_NA = True
        for i in range(self.nb_elements_per_line) :
            value = obi_column_get_obiseq_with_elt_idx_in_view(self.view.pointer, (self.pointer)[0], line_nb, i)
            if obi_errno > 0 :
                raise IndexError(line_nb)
            if value == OBISeq_NA :
                value_in_result = None
            else :
                try:
                    value_in_result = <bytes> value
                finally:
                    free(value)    
            result[self.elements_names[i]] = value_in_result
            if all_NA and (value_in_result is not None) :
                all_NA = False
        if all_NA :
            result = None
        return result

    cpdef set_item(self, index_t line_nb, str element_name, object value):
        cdef bytes value_b
        
        if value is None :
            value_b = OBISeq_NA
        elif isinstance(value, bytes) :
            value_b = value
        elif isinstance(value, str) :
            value_b = str2bytes(value)
        else:
            raise TypeError('Sequence value must be of type Bytes, Str or None')
        
        if obi_column_set_obiseq_with_elt_name_in_view(self.view.pointer, (self.pointer)[0], line_nb, str2bytes(element_name), value_b) < 0:
            raise Exception("Problem setting a value in a column")
    
#    cpdef align(self, ):        # TODO
#        raise Exception("Columns with multiple sequences per line can't be aligned")    # TODO discuss

