#cython: language_level=3

from obitools3.utils cimport bytes2str, str2bytes

from .capi.obiview cimport NUC_SEQUENCE_COLUMN, \
                           ID_COLUMN, \
                           DEFINITION_COLUMN, \
                           QUALITY_COLUMN

cdef class OBI_Seq(dict) :

    def __init__(self, object id, object seq, object definition=None) :
        self.set_id(id)
        self.set_sequence(seq)
        if definition is not None :
            self.set_definition(definition)
        
    cpdef set_id(self, object id) :
        self.id = id
        self[bytes2str(ID_COLUMN)] = id
        
    cpdef get_id(self) :
        return self.id
            
    cpdef set_definition(self, object definition) :
        self.definition = definition
        self[bytes2str(DEFINITION_COLUMN)] = definition
        
    cpdef get_definition(self) :
        return self.definition

    cpdef get_sequence(self) :
        return self.sequence

    def __str__(self) :
        return self.sequence    # or not


cdef class OBI_Nuc_Seq(OBI_Seq) :
    
    cpdef set_sequence(self, object sequence) :
        self.sequence = sequence
        self[bytes2str(NUC_SEQUENCE_COLUMN)] = sequence

    cpdef set_quality(self, object quality) :
        self.quality = quality
        self[bytes2str(QUALITY_COLUMN)] = quality

    cpdef get_quality(self) :
        return self.quality

#    cpdef str reverse_complement(self) :    TODO in C ?
#        pass


cdef class OBI_Nuc_Seq_Stored(OBIView_line) :

    # TODO store the str version of column name macros

    cpdef set_id(self, object id) :
        self[bytes2str(ID_COLUMN)] = id
        
    cpdef object get_id(self) :
        return self[bytes2str(ID_COLUMN)]
            
    cpdef set_definition(self, object definition) :
        self[bytes2str(DEFINITION_COLUMN)] = definition

    cpdef object get_definition(self) :
        return self[bytes2str(DEFINITION_COLUMN)]

    cpdef set_sequence(self, object sequence) :
        self[bytes2str(NUC_SEQUENCE_COLUMN)] = sequence

    cpdef object get_sequence(self) :
        return self[bytes2str(NUC_SEQUENCE_COLUMN)]
    
    cpdef set_quality(self, object quality) :
        if (type(quality) == list) or (quality is None) :
            self[bytes2str(QUALITY_COLUMN)] = quality
        else :  # Quality is in str form
            (((self.view).columns)[bytes2str(QUALITY_COLUMN)]).set_str_line(self.index, quality)

    cpdef object get_quality(self) :
        return self[bytes2str(QUALITY_COLUMN)]

    cpdef object get_str_quality(self) :
        return ((self.view).columns)[bytes2str(QUALITY_COLUMN)].get_str_line(self.index)

#    def __str__(self) :
#        return self[bytes2str(NUC_SEQUENCE_COLUMN)]    # or not
    
#    cpdef str reverse_complement(self) :    TODO in C ?
#        pass

    # TODO static method to import?
    
